package HotelService.database;

import HotelService.*;

import java.math.*;
import java.util.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;



public abstract class GenericHotelDB implements HotelDB {

    protected Connection conn = null;
    protected ResultSet rs = null;
    protected Statement stmt = null;
    protected PreparedStatement pstmt = null;
    protected String dbUrl = Globals.dbUrl; // "jdbc:mysql://localhost/hotel?user=root";

    protected GenericHotelDB() throws DatabaseException{
	connect();
    }

    // this should connect to the database and initialize stmt
    public abstract void connect() throws DatabaseException;

    public void close() throws DatabaseException{
	try{
	    conn.close();
	} catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}
    }

    // main methods
    public int createAccount(String firstName,
			     String surName,
			     String password)
	throws DatabaseException {
	
	try{

	    // pstmt needs a BigDecimal instead of BigInteger
	    


	    String sql = "INSERT INTO Client (firstName,surName,password) VALUES (?,?,?)";
	    pstmt = conn.prepareStatement(sql);

	    pstmt.setString(1,firstName);
	    pstmt.setString(2,surName);
	    pstmt.setString(3,password);
	    
	    // execute the query
	    pstmt.executeUpdate();
	    
	    // get the id of the row we just added
	    sql = "SELECT max(id) FROM Client";

	    rs = stmt.executeQuery(sql);
	    rs.next();
	    int rc = rs.getInt(1);
	    
	    return rc;

	} catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}

	
    }

    public int createReservation(int clientId, 
				 int hotelId,
				 String hotelName,
				 int roomSize,
				 Date startDate,
				 int nights
				 )  throws DatabaseException
    {
	
	try{
	    

	    // we need an SQL date
	    java.sql.Date startDateSql = new java.sql.Date(startDate.getTime());

	    String sql = "INSERT INTO Reservations (clientId,hotelId, hotelName, roomSize,startDate,nights) VALUES (?,?,?,?,?,?)";
	    
	    pstmt = conn.prepareStatement(sql);

	    pstmt.setInt(1,clientId);
	    pstmt.setInt(2,hotelId);
	    pstmt.setString(3,hotelName);
	    pstmt.setInt(4,roomSize);
	    pstmt.setDate(5,startDateSql);
	    pstmt.setInt(6,nights);



	    // execute the query
	    pstmt.executeUpdate();
	    
	    // get the id of the row we just added
	    sql = "SELECT max(reservationId) FROM Reservations";

	    rs = stmt.executeQuery(sql);
	    rs.next();
	    int rc = rs.getInt(1);
	    
	    return rc;


	} catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}
    }
    
    public Vector listHotels(String city)  throws DatabaseException{

	try{
	    
	    String sql = "SELECT * FROM Hotel WHERE city="+ "'" +city + "'";
	    rs = stmt.executeQuery(sql);
	    
	    Vector results = new Vector();

	    while(rs.next()){
		// creating and adding Hotels to results
		Hotel hotel = new Hotel();
		hotel.setId(new BigInteger(rs.getString("id")));
		hotel.setName(rs.getString("name"));
		hotel.setStars(new BigInteger(rs.getString("stars")));
		hotel.setStreet(rs.getString("street"));
		hotel.setZip(new BigInteger(rs.getString("zip")));
		hotel.setCity(rs.getString("city"));
		results.addElement(hotel);
	    }

	    

	    // hotels are now listed, we have to set their prices
	    for(int i=0;i<results.size();i++){
		Hotel hotel = (Hotel) results.elementAt(i);
		int hotelId = hotel.getId().intValue();
		
		sql="SELECT * FROM HotelPrices WHERE hotelId="+hotelId;
	      
		
		rs=stmt.executeQuery(sql);
		Vector prices = new Vector();
		while(rs.next()){
		    //add prices to the prices vector
		    RoomPrices roomPrices = new RoomPrices();
		    roomPrices.setSize(new BigInteger(rs.getString("roomSize")));
		    roomPrices.setPrice(rs.getFloat("roomPrice"));
		    roomPrices.setTotalRooms(new BigInteger(rs.getString("totalRooms")));
		    prices.addElement(roomPrices);
		}

		// prices is now complete, we can recopy the values to the hotel
		RoomPrices [] tempPrices = new RoomPrices[prices.size()];
		
		for(int j=0;j<prices.size();j++){
		    tempPrices[j]=(RoomPrices)prices.elementAt(j);
		} 
		hotel.setRoomPrices(tempPrices);
	    }
	
	    return results;

	} catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}
	
	
    }

    public Vector listReservation(int clientId, String password)  throws DatabaseException{

	try{
	    
	    String sql = "SELECT * FROM Reservations WHERE clientId="+clientId;
	    rs = stmt.executeQuery(sql);
	    Vector results = new Vector();

	    while(rs.next()){
		// creating and adding Reservations to results
		Reservation reservation = new Reservation();
		reservation.setId(new BigInteger(rs.getString("reservationId")));
		reservation.setClientId(new BigInteger(rs.getString("clientId")));
		reservation.setHotelId(new BigInteger(rs.getString("hotelId")));
		reservation.setHotelName(rs.getString("hotelName"));

		reservation.setRoomSize(new BigInteger(rs.getString("roomSize")));
                java.util.Date sDate = rs.getDate("startDate");
                Calendar startDate = new java.util.GregorianCalendar();
                startDate.setTime(sDate);
		reservation.setStartDate(startDate);
		reservation.setNights(new BigInteger(rs.getString("nights")));
				
		results.addElement(reservation);
	    }
	
	    return results;    
	} catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}
	
	
    }
    
    public void cancelReservation(int reservationId)  throws DatabaseException{
	try{

	    String sql = "DELETE FROM Reservations WHERE reservationId="+reservationId;
	    stmt.executeUpdate(sql);

	} catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}


    }
    
    // helper methods

    public boolean checkPassword(int id, String password)  throws DatabaseException{
	System.out.println("checkPassword...1");
	String sql = "SELECT * FROM Client WHERE id=" + id + " AND password=" + "'" + password + "'";
	System.out.println("checkPassword...2");
	
	try{
	    System.out.println("checkPassword...3");
	    rs = stmt.executeQuery(sql);
	    System.out.println("checkPassword...4");
	    if(rs.next()){
		System.out.println("checkPassword...5");
		return true;
	    } else{
		System.out.println("checkPassword...6");
		return false;
	    }
	}catch(Exception ex){
	    System.out.println("checkPassword...7");
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}
    }

    public boolean checkAvailaibleRooms(int roomSize,
					int hotelId,
					Date startDate,
					int nights
					)  throws DatabaseException{
	
	// find a simple algo that calculates the number of rooms that are left
	// at the date startDatem for the number of nights specified

	System.out.println("checkAvailaibleRooms...1");

	try{
	    String sql = "SELECT totalRooms FROM HotelPrices WHERE hotelId="+hotelId +" AND roomSize=" + roomSize;

	    System.out.println("checkAvailaibleRooms...2");

	    rs = stmt.executeQuery(sql);

	    System.out.println("checkAvailaibleRooms...3");
	    
	    if(!rs.next()){
		System.out.println("checkAvailaibleRooms...4");
		return false;
	    }
	    // else, everything is ok, we can get the value and go on
	    
	    System.out.println("checkAvailaibleRooms...5");
	    int totalRooms = rs.getInt(1);
	    int rooms = 0;

	    System.out.println("checkAvailaibleRooms...6");
	    sql = "SELECT * FROM Reservations";

	    System.out.println("checkAvailaibleRooms...7");
	    rs = stmt.executeQuery(sql);

	    System.out.println("checkAvailaibleRooms...8");

	    while(rs.next()){
		System.out.println("checkAvailaibleRooms...9");
		int currentRoomSize = rs.getInt("roomSize");

		System.out.println("checkAvailaibleRooms...10");
		if(roomSize != currentRoomSize){
		    System.out.println("checkAvailaibleRooms...11");
		    break;
		}

		System.out.println("checkAvailaibleRooms...12");
		// else, we increment the number of rooms taken during the period [startdate,startdate+nights]
		
		Date currentStartDate = rs.getDate("startDate");
		int currentNights = rs.getInt("nights");
		
		System.out.println("checkAvailaibleRooms...13");
		long currentEndDateMS = currentStartDate.getTime() + currentNights * 24 * 60 * 60 * 1000;
		Date currentEndDate = new Date(currentEndDateMS);
		
		System.out.println("checkAvailaibleRooms...14");
		// same thing for our date
		long endDateMS = startDate.getTime() + nights * 24 * 60 * 60 * 1000;
		Date endDate = new Date(endDateMS);

		System.out.println("checkAvailaibleRooms...15");

		// now, we can work
		// basically, we increment if the current reservation
		// 1 - starts before and ends after
		// 2 - starts  during
		// 3 - ends during

		if(startDate.before(currentStartDate) && endDate.after(currentEndDate)){
		    System.out.println("checkAvailaibleRooms...16");
		    rooms++;
		    continue;
		}

		if(startDate.after(currentStartDate) && startDate.before(currentEndDate)){
		    System.out.println("checkAvailaibleRooms...17");
		    rooms++;
		    continue;
		}

		if(endDate.after(currentStartDate) && endDate.before(currentEndDate)){
		    System.out.println("checkAvailaibleRooms...18");
		    rooms++;
		    continue;
		}
		
		
		
		
	    }

	    if(rooms < totalRooms){
		System.out.println("checkAvailaibleRooms...19. rooms="+rooms+"/"+totalRooms);
		return true;
	    } else{
	    
		System.out.println("checkAvailaibleRooms...20");
		return false;		
	    }
	    
	    
	}catch(Exception ex){
	    throw new DatabaseException(new BigInteger("1"),ex.getMessage());
	}

    }


    // to allow Factory to change the host string
}
