package YellowPagesClient;

import java.util.*;
import java.math.*;


public class SimpleCompiler{


    // this table maps keywords to variable types
    // the keywords will be used in the lexical analyzer
    // the variable types will be used ???

    String [][] keywords = {
	{"categoryName","String"},
	{"zip","Integer"},
	{"street","String"},
	{"city","String"}
    };





    // state table
    // abbrevs. <op> = operator (and, or), <k> = keyword *categoryName, zip, street, city, <v> = value( anything inside quotes)
    // col are    (  )  <op>  <k>  =  '  <v>
    // the value of states[a,b] is the state that the automate should go from state a after receiving b
    // -1 means ERROR

    // for lisp like syntax
    // int [][] states = {
    //{ 1, -1, -1, 2, -1, -1, -1 }, // state A
    //{ 1, -1, 7, 2, -1, -1, -1 }, // state B
    //{ -1, -1, -1, -1, 3, -1, -1 }, // state C
    //{ -1, -1, -1, -1, -1, 4, -1 }, // state D
    //{ -1, -1, -1, -1, -1, -1, 5 }, // state E
    //{ -1, -1, -1, -1, -1, 6, -1 }, // state F
    //{ -1, 9, -1, -1, -1, -1, -1 }, // state G
    //  { 8, -1, -1, -1, -1, -1, -1 }, // state H
    //{ 8, -1, 7, 2, -1, -1, -1 }, // state I
    //{ 8, 9, -1, -1, -1, -1, -1 } // state J
    //};

    // for sql like syntax
    int [][] states = {
	{ 1, -1, -1, 2, -1, -1, -1 }, // state A
	{ 1, -1, -1, 2, -1, -1, -1 }, // state B
	{ -1, -1, -1, -1, 3, -1, -1 }, // state C
	{ -1, -1, -1, -1, -1, 4, -1 }, // state D
	{ -1, -1, -1, -1, -1, -1, 5 }, // state E
	{ -1, -1, -1, -1, -1, 6, 5 }, // state F
	{ -1, 6, 7, -1, -1, -1, -1 }, // state G
	{ 1, -1, -1, 2, -1, -1, -1 } // state H
    };

    // strings to number mapping
    // in the following table, each input string is at 
    // the position used in the preceding table.
    // example : string "(" is at position 0 of this table, 
    // which means that in a line x (state x) of the states table,
    // states[x][0] will be the index of the new state when the machine
    // is in state x and has an input of 0, which is "("

    String [] inputs = {"(", ")", "<op>", "<k>", "=", "\'", "<v>"};


    // same as above, but for row names

    // for lisp like syntax
    //String [] stateNames = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J"};

    // for sql like syntax
    String [] stateNames = {"A", "B", "C", "D", "E", "F", "G", "H"};

    // logical operators
    String [] logOps = {"and","or"};

    char [] separators = {'(',')','\'','='};

    // constants
    protected final int startState = 0;

    // for lisp like syntax
    //protected final int[] finalStates = {6,9};

    // for sql like syntax
    protected final int[] finalStates = {6};
    

    public SimpleCompiler(){
	
    } // SimpleCompiler


    public Vector lexAnalyzer(String someString) throws Exception{
	Vector result = new Vector();



	// the following 2 variables are used to determine whether we're
	// reading a value or logOp.
	// if we're inside quotes, we assume we are reading a value.

	int quotes = 0; 
	boolean readValue = false; 


	// loop until the string is completly parsed
	int i = 0;

	while(i < someString.length()){
	    
	    char currentChar = someString.charAt(i);
	  
	    
	    if(currentChar   == '\''){
		quotes++;
		System.out.println("number of quotes: "+quotes);
	    } 

	    if(quotes % 2 == 0){
		System.out.println("do not read a value");
		readValue = false;
	    } else{
		System.out.println(" read a value");
		readValue = true;
	    }


	    // now let's analyze the Character
	    
	    // if its a spacem skip it
	    if(isSpace(currentChar)){
		System.out.println("char "+i+" is a space, lets skip" );
		i++;
		continue;
	    }
	    
	    // if its a separator, add it
	    if(isSeparator(currentChar)){
		System.out.println("char "+i+" is a separator" );
		Character sep = new Character(currentChar);
		String sepString = sep.toString();

		// add the separator to the results Vector
		result.addElement(sepString);
		
		i++;
		continue;
	    }
	    
	    // if its a letter or a digit, recognize the whole word/number
	    if(isDigitOrLetter(currentChar)){
		System.out.println("char "+i+" is a digit or a letter" );
		
		String word = "";
		
		// lets try to recognize the word
		while( ( readValue && isSpace(currentChar)) || ( (!isSpace(currentChar)) && (!isSeparator(currentChar)) && (isDigitOrLetter(currentChar)))  ){
		    
		    Character temp = new Character(currentChar);
		    word = word+temp.toString();
		    
		    i++;
		    currentChar = someString.charAt(i);
		}

		
		// adding element to result
		result.addElement(word);
		

		// to complete, dont forget i++ etc
		continue;
	    }
	    
	    // Everything else
	    System.out.println("char "+i+" is nothing");
	    i++;
	    throw new Exception("Parse Error at char "+i);
	}
	
	return result;
    }

    // input must be a Vector of Strings
    public void syntaxAnalyzer(Vector inputsVector) throws Exception{

	
	int state = startState; // initial state is A (0)
	int counter = 0; // must be 0 at the end too, otherwise ERROR

	
	// the following 2 variables are used to determine whether we're
	// reading a value or logOp.
	// if we're inside quotes, we assume we are reading a value.

	int quotes = 0; 
	boolean readingValue = true; 

	/////////////////////////////

	System.out.println("Starting with state " + stateNames[state]);

	for(int i = 0 ; i < inputsVector.size() ; i++){
	    String input = (String) inputsVector.elementAt(i);

	    // check to see if the input is a separator, an operator or a value
	    String inputCode = "";

	    if(isSeparator(input)){
		inputCode = input;

		// mise a jour du compteur
		if(input.equals("(")){
		    counter++;
		}
		
		if(input.equals(")")){
		    counter--;
		}

		if(input.equals("\'")){
		    quotes++;
		}

		System.out.println("counter: "+counter+" quotes: "+quotes);

	    } else{
		
		if(quotes % 2 == 0){
		    readingValue = false;
		} else{
		    readingValue = true;
		}

		if(isLogOp(input)){
		    System.out.println("<op>");
		    inputCode = "<op>";
		} else{
		    if(!readingValue && isKeyword(input)){
			System.out.println("<k>");
			inputCode = "<k>";
		    } else{
			if(isValue(input)){
			    System.out.println("<v>");
			    inputCode = "<v>";
			} else{
			    // this shouldnt happen, but in case....
			    System.out.println("weird, the lex analyzer didnt do its job, ERROR...");
			}
		    }
		}
	    }
	    
	    

	    int intputIndex = getInputIndex(inputCode);

	    int nextState = states[state][intputIndex];
	    if(nextState == -1){
		throw new Exception("state "+stateNames[state]+ " doesnt accept input " + input);
	    }
	    
	    state = nextState;
	    System.out.println("state is now " + stateNames[state]);

	} // for
	
	// the input vector is finished, lets check if it stopped at a consistent state (6 or 9)
	System.out.println("checking final state...");
	if(checkFinalState(state) ){
	    System.out.println("OK");
	} else{
	    throw new Exception("The string is not recognized. Final state is NOT OK: "+state);
	}

	// check the counter 
	System.out.println("checking counter (parenthesis)...");
	if(counter == 0 ){
	    System.out.println("OK");
	} else{
	    throw new Exception("There are "+counter + " mismatched parenthesis");
	}

    }

    int getKeywordType(String kw){
	System.out.println("getKwType()");
	int rc = -1;
	int i = 0;
	while(i < keywords.length){
	    
	    if(kw.equals(keywords[i][0])){
		return i;
	    }
	    i++;
	    
	} // while

	return rc;
    } // getKeywordType()

    public String semAnalyzer(Vector inputsVector) throws Exception{

	// the result vector
	Vector resultVector = new Vector();	
       
	// the result string, generated from the vector
	String result="";


	// for the loop
	int i = 0;
	int max = inputsVector.size();

	while (i < max){
	    
	    String item = (String) inputsVector.elementAt(i);
	    
	    System.out.println("reading item: "+item); // debug

	    
	    if(isSeparator(item)){
		resultVector.addElement(" ");
		resultVector.addElement(item);
		resultVector.addElement(" ");
	    } else{

		if(isKeyword(item)){
		    String value = (String) inputsVector.elementAt(i+3);
		    int indexType=getKeywordType(item);
		    String type = keywords[indexType][1];

		    if(indexType == -1){
			System.out.println("-1 received for type");
		    } else{
			
			System.out.println("item: " +item + " value: "+value);
			if(type.equals("Integer")){
			    try{
				
				
				BigInteger number = new BigInteger(value);
				
				resultVector.addElement(item);
				resultVector.addElement("=");
				resultVector.addElement(value);
				resultVector.addElement(" ");
				
				i = i + 4;
			    } catch(Exception ex){
				throw new Exception(item+" must be an integer");			    
			    } // catch
			} else{
			    resultVector.addElement(item);
			    resultVector.addElement("=");
			    resultVector.addElement("'");
			    resultVector.addElement(value);
			    resultVector.addElement("'");
			    i = i + 4;
			}
		    }
		    
		} else{
		    // it should be an operator
		    resultVector.addElement(" ");
		    resultVector.addElement(item);
		    resultVector.addElement(" ");
		}

	    } // else isSeparator

	    i++;
	}
	
	
	// returning result
	result="";
	for(int j = 0 ; j < resultVector.size() ; j++){
	    String temp = (String) (resultVector.elementAt(j));
	    result=result + temp;
	} // for

	return result;

    } // semAnalyzer


    // semantic analyzer for lisp like syntax
    // recursive method !! first call with 0, inputsVector.size() -1
    public String sqlTransformer(Vector inputsVector, int start, int end) throws Exception{

	System.out.println("debug: sqlTransformer(): " + start + " " + end);

	String result = "";
	String firstItem = (String) inputsVector.elementAt(start);
	String secondItem = (String) inputsVector.elementAt(start+1);

	// first, we are supposed to check the type and stuff
	// 
	// transformTypes(Vector inputsVector)



	// now we can rewrite the expression

	// 1) Simple case.
	// if the first or second item is a keyword, it means its a simple element, so 
	// we can just print it
	

	if( (isKeyword(firstItem)) || (isKeyword(secondItem))){
	    for(int i = start ; i <= end ; i++){
		String temp = (String) inputsVector.elementAt(i);
		result = result + temp;
	    } // for
	    System.out.println("partial result: " + result);
	    return result;
	} // if

	// 2) Simple recursive case
	// if the 2nd item is a (, then we just match the parenthese, and 

	if(secondItem.equals("(")){
	    int matchEnd = matchParenthese(inputsVector,start+1, end-1);
	    result = result + sqlTransformer(inputsVector, start+1,matchEnd);
	} // if

	// 3) Complex recursive case
	// if 2nd item is an operator
	// then we do complex stuf :p

	if(isLogOp(secondItem)){
	    System.out.println("logOp");

	    // we count the number of opening parenthesis
	    int nbrOfPar = countOpeningParentheses(inputsVector,start+2,end-1);
	    
	    System.out.println("opening parenthesis: " + nbrOfPar);
	    if(nbrOfPar == 1){
		int matching = matchParenthese(inputsVector,start+2,end-1);
		
		for(int i = start +2 ; i <= end -1 ; i++){
		    String temp = (String) inputsVector.elementAt(i);
		    result = result + " " + temp;
		} // for
		System.out.println("partial result: " + result);
		return result;
	    } else{ // there are at least 2 elements

		System.out.println("debug: else, there are at least 2 operands");

		boolean or = false;

		int i = start+2;
		while( i <= end -1){
		    String temp = (String) inputsVector.elementAt(i);
		    
		    if(temp.equals("(")){
			int endIndex = matchParenthese(inputsVector,i,end-1);
			
			if(or){
			    result = result + " OR ";
			    or = true;
			}

			result = result + sqlTransformer(inputsVector,i,endIndex);

			// ugly, but...
			i = endIndex;
		    } //if

		    i++;
		    
		} // for
	    }

	}
	    return result;
    }
	
	
	protected int countOpeningParentheses(Vector v, int index, int endindex) {
	
	int counter = 0;

	int i = index;

	boolean closed = true;

	while ( i <= endindex){
	    String temp = (String) v.elementAt(i);

	    if(temp.equals(")")){
		closed = true;
	    }

	    if(temp.equals("(")){
		// dont update counter unless closed is true - work around
		if(closed){
		    
		    counter++;
		    closed = false;
		}
	    } // if
	    i++;
	} // while
	return counter;

    }

    //returns the index of the item that matches parenthese at index index
    protected int matchParenthese(Vector v, int index, int endindex) throws Exception{
	
	int counter = 0;
	int i = index +1;
	while(i <= endindex ){
	    
	    String temp = (String) v.elementAt(i);
	    
	    if(temp.equals("(")){
		counter++;
	    } // if
	    
	    if(temp.equals(")")){
		counter--;
	    } // if

	    // check whether the current string was the matching parenthesis

	    if(counter == -1){
		System.out.println("matching for "+index+" is "+i);
		return i;
	    } // if

	    i++;
	} // while
	

	// if we arrive here, send an exception
	throw new Exception("There was no matching parenthese");

    } // matchParenthese

    // Character-helpers
    protected boolean isSpace(char someCharacter){
	if (someCharacter == ' '){
	    return true;
	}
	else{
	    return false;
	}
    }


    protected boolean isSeparator(char someCharacter){
	boolean rc = false;
	
	for(int i = 0 ; i<separators.length ; i++){
	    if(someCharacter == separators[i]){
		return true;
	    }
	}

	return rc;       
    }

    protected boolean isDigitOrLetter(char someCharacter){
	if(Character.isDigit(someCharacter) || Character.isLetter(someCharacter)){
	    return true;
	}
	else{
	    return false;
	}
    }

    // String-helpers

    // returns -1 if input doesnt exist in the table
    protected int getInputIndex(String inputString){
		
	int rc = -1;
	int i = 0;

	while(i < inputs.length){
	    String temp = inputs[i];
	    if(inputString.equals(temp)){
		return i;
	    } else{
		i++;
	    }
	}

	return rc;
	
    }

    protected boolean isSpace(String someString){
	if (someString.equals(" ")){
	    return true;
	}
	else{
	    return false;
	}
    }


    protected boolean isSeparator(String someString){
	boolean rc = false;
	
	for(int i = 0 ; i<separators.length ; i++){

	    Character sep = new Character(separators[i]);
	    String sepString = sep.toString();

	    if(someString.equals(sepString)){
		return true;
	    }
	}

	return rc;       
    }

    protected boolean isLogOp(String word){
	boolean rc = false;
	
	for(int i = 0 ; i<logOps.length ; i++){
	    if(word.equals(logOps[i])){
		return true;
	    }
	}

	return rc;       
    }

    protected boolean isKeyword(String word){
	boolean rc = false;

	for(int i = 0 ; i<keywords.length ; i++){
	    if(word.equals(keywords[i][0])){
		return true;
	    }
	}

	return rc;
    }

    protected boolean isValue(String word){
	boolean rc = true;

	return rc;

    }


    //returns true if state is a final state
    protected boolean checkFinalState(int state){
	boolean rc = false;
	
	for(int i = 0; i < finalStates.length ; i++){
	    if(state == finalStates[i]){
		return true;
	    } // if
	} // for

	
	return rc;

    }

    public String parseExpression(String expression) throws Exception{
	
	
	Vector result = lexAnalyzer(expression);
	syntaxAnalyzer(result);
	return semAnalyzer(result);
    }

    public static void main(String [] args){

	try{
	
	    SimpleCompiler compiler = new SimpleCompiler();
	    
	    
	    //Vector result = compiler.lexAnalyzer("((or((city='truc')(street='machin'))))");
	    Vector result = compiler.lexAnalyzer("((city='city 1' or city='city 2') and (zip='75000' or categoryName='hotel' or categoryName='restaurant') )");
	    //	    Vector result = compiler.lexAnalyzer("city='truc'");
	    //	    Vector result = compiler.lexAnalyzer("((or((city='truc'))))");
	    
	    System.out.println("RESULT: ");
		
	    for(int i = 0 ; i <result.size(); i++){
		
		String temp = (String) result.elementAt(i);
		
		System.out.println(temp);
	    }

	    System.out.println("Lets call now the syntax analyzer");
	    compiler.syntaxAnalyzer(result);

	    System.out.println("RESULT 2: ");
		
	    for(int i = 0 ; i <result.size(); i++){
		
		String temp = (String) result.elementAt(i);
		
		System.out.println(temp);
	    }

	    //System.out.println(compiler.sqlTransformer(result,0,result.size() -1));
	    
	    System.out.println(compiler.semAnalyzer(result));

	} catch(Exception ex){
	    System.out.println("Exception msg: " + ex.toString());
	}
	

    }

}
