package CoreBroker;

import java.util.HashMap;
import java.util.Vector;

public abstract class CLBaseClass 
{

   protected HashMap options;
        // options are arguments on the command line specified with an initial "--"
   protected String[] params; 
   protected int nParams;
   protected String paramsDescription;
        // parameters are arguments on the command line specified without an initial "--" 
        // and they are at the end of the command line after all the options
   protected static final int ATLEASTONE = -1;
   
   public CLBaseClass() {
        options = new HashMap();
        nParams = setNParams();
        paramsDescription = setParamsDescription();
        params = null;
        options.put("help",new CLOptionDescr("help",0,"help"));
   }
   
   protected abstract int setNParams();
   
   protected abstract String setParamsDescription();
   
   protected abstract void addOptions();
    
   protected boolean parseArgs(String args[]) {
        int i=0;
        //boolean exit = false;
        boolean error = false;
        int errorPosition = -1;
        
        while (i<args.length && !error)
        {            
            if (args[i].startsWith("--"))
            {
               String optionName = args[i].substring(2);
               
               if (options.containsKey(optionName))
               {
                   if (optionName.equals("help"))
                   {
                       // if option help is contained in the command line exit and print help
                       error = true;
                   }
                   else 
                   {
                      CLOptionDescr opt = (CLOptionDescr)options.get(optionName);
                      // verify if the option has the right number of parameter
                      if (opt.numParams == 0)
                      {
                          // if option must have 0 and is specified, the value is TRUE 
                            opt.value = Boolean.TRUE;
                            options.put(optionName,opt);
                            i++;                            
                      } 
                      else if (opt.numParams == ATLEASTONE)
                      {
                          // if unlimited number of parameter for this option
                          // read its parameters until the end of the command line
                          // or new option
                            Vector values = new Vector();
                            boolean noMoreParams = false;
                            i+=1;
                            while (i<args.length && !noMoreParams)
                            {
                               if (args[i].startsWith("--"))
                               {
                                    noMoreParams = true;
                               } 
                               else
                               {
                                    values.add(args[i]);
                                    i+=1;
                               }
                            }
                            
                            
                            if (values.size() == 0)
                                // if no parameters for this option generate error ...
                            {
                                error = true;
                                errorPosition = i-1;
                            } 
                            else 
                            {
                                // ... else store all parameters values found for this option
                                opt.value = values;
                                options.put(optionName, opt);                          
                            }
                      } 
                      else if (opt.numParams == 1)
                      {
                            boolean valueIsOK = false;
                            if (opt.possibleValues == null)
                                valueIsOK = true;
                            else 
                                for (int j=0; j<opt.possibleValues.length; j++)
                                    if (opt.possibleValues[j].equals(args[i+1]))
                                        valueIsOK =true;

                            if (valueIsOK && (i+1 < args.length))
                            {
                                opt.value = args[i+1];
                                options.put(optionName, opt);
                                i+=2;
                            } else {
                                error = true;
                                errorPosition = i;
                            }
                      }
                      else //if (opt.numParams is a fix number but >=2
                      {
                          // read its parameters until the end of the command line
                          // or new option or opt.numParams is reached
                            Vector values = new Vector();
                            boolean noMoreParams = false;
                            int paramsFound = 0;
                            i+=1;
                            while (i<args.length && !noMoreParams 
                                    && paramsFound < opt.numParams)
                            {
                               if (args[i].startsWith("--"))
                               {
                                    noMoreParams = true;
                               } 
                               else
                               {
                                    values.add(args[i]);
                                    i+=1;
                                    paramsFound++;
                               }
                            }
                            
                            
                            if (values.size() != opt.numParams)
                                // if no parameters for this option generate error ...
                            {
                                error = true;
                                errorPosition = i-1;
                            } 
                            else 
                            {
                                // ... else store all parameters values found for this option
                                opt.value = values;
                                options.put(optionName, opt);                          
                            }
                      }                      
                   }
               }
               else // the specified option is not in the list of those allowed
               {
                    error = true;
                    errorPosition = i;
               }
            } 
            else 
                // the argument has no initial "--": it's a final parameter
                // in the command line: need to verify that no further options
                // are specified in command line
            {
                boolean noOptionsInParams = true;
                boolean wrongParameterNumber = false;
                
                // check if option among parameters
                int indexFirstParam = i;
                for (int k=indexFirstParam; k<args.length; k++)
                    if (args[k].startsWith("--"))
                    {
                        noOptionsInParams = false;
                    }
                
                // check if the number of parameters is right
                wrongParameterNumber = (nParams != (args.length-indexFirstParam)) && (nParams != ATLEASTONE);
                             
                
                // store parameters in array                
                if (noOptionsInParams && !wrongParameterNumber)
                {
                    params = new String[args.length-indexFirstParam];
                    for (int k=indexFirstParam; k<args.length; k++)
                        params[k-indexFirstParam] = args[k];
                    i = args.length;
                } else {
                    errorPosition = i; 
                    error = true;
                }
            }
        }       
        
        if (error) {
            if (errorPosition != -1) //error position is set if an error occurred in options, not set if error in parameters
                Syslog.writelog("Error in argument: "+args[errorPosition]);
            help();            
            return false;
        } else if (params == null)
            if(nParams !=0) {
                // if the number of required parameters is not zero and the end
                // of thecommand line has been reached there is an error
                Syslog.writelog("No parameters specified ");
                help();
                return false;
            } else if (nParams == 0 && args.length == 0) {
                // if the number of required parameters is zero and 
                // the command line is only made by the command name
                // print the help                
                help();
                return false;                
            } else
                return true;
        else            
            return true;
    }

   protected String[] getArrayStringOption(String optionName)
   {
       String[] values;
       CLOptionDescr opt = (CLOptionDescr)options.get(optionName);
       Vector vect = (Vector)opt.value;
       if (vect != null && vect.size()  > 0)
       {
            values = new String[vect.size()];
            vect.copyInto(values);           
            return values;
       }
       else
           return null;
   }   
   
   protected String getStringOption(String optionName)
   {
       CLOptionDescr opt = (CLOptionDescr)options.get(optionName);
       return (String)opt.value;
   }

   protected boolean getBooleanOption(String optionName)
   {
       CLOptionDescr opt = (CLOptionDescr)options.get(optionName);
       return ((Boolean)opt.value).booleanValue();
   }

   
   abstract protected void setValues();
   
   protected void help()
   {
        System.out.println("\nusage: "+ this.getClass().getName() +" [options] "+((paramsDescription!=null)?paramsDescription:""));
        System.out.println("Options:");
        Object[] optionNames = options.keySet().toArray();
        
        for (int i=0; i<optionNames.length; i++)
        {
            System.out.print("\t--"+optionNames[i]);
            if (((CLOptionDescr)options.get(optionNames[i])).numParams >= 1) 
            {
                // print default value and possible values
                String[] possibleValues = ((CLOptionDescr)options.get(optionNames[i])).possibleValues;
                if (possibleValues == null)
                    for (int j=0; j<((CLOptionDescr)options.get(optionNames[i])).numParams; j++)
                    System.out.print(" <optionvalue>");
                else
                {
                    System.out.print("\t[ ");
                    for (int j=0; j<possibleValues.length-1; j++)
                        System.out.print(possibleValues[j]+" | ");
                    System.out.print(possibleValues[possibleValues.length-1]);
                    System.out.print(" ]");
                }
                if (((CLOptionDescr)options.get(optionNames[i])).defaultValue != null)
                    System.out.println("  ( default = \""+ ((CLOptionDescr)options.get(optionNames[i])).defaultValue+"\" )");
                else
                    System.out.println();
            }
            else 
                System.out.println();
            
            // split description in multiple lines
            String descr = ((CLOptionDescr)options.get(optionNames[i])).descr;
            int descrLength = descr.length();
            int firstIndex = 0;
            while (firstIndex < descrLength)
            {
                int lastIndex = firstIndex+80;
                if (lastIndex > descrLength)
                    lastIndex = descrLength;
                String subDescr = descr.substring(firstIndex, lastIndex);
                int spaceIndex = subDescr.lastIndexOf(' ');
                if (spaceIndex == -1 || lastIndex == descrLength) 
                    spaceIndex = descrLength;
                else
                    spaceIndex += firstIndex;
                System.out.println("\t\t"+descr.substring(firstIndex, spaceIndex));
                firstIndex = spaceIndex+1;
            }
        }
        System.out.println("\n");
   }
    
   public boolean run(String args[])
   {
       addOptions();
       if (parseArgs(args))
       {
            setValues();
            return true;
       } else
           return false;
   }

}
