#include <stdio.h>
#include <netinet/in.h>
#include <linux/sockios.h>
#include "CoreBroker_IFUtils.h"
#include "iwlib.h"
#include "apm.h"


JNIEXPORT void JNICALL Java_CoreBroker_IFUtils_iwconfig2
  (JNIEnv * env, jobject jTmp) {
    
    const char* ifname = "eth0";
    struct wireless_info	info;
    int rc;  
    int skfd;		

    int fd;
    apm_info apm;
    int result;

    if((skfd = iw_sockets_open()) < 0)
    {
      exit(-1);
    }

    //  iw_print_version_info(ifname);
    iw_usage();

    print_info(skfd, "eth0", NULL, 0);
 
    rc = get_info(skfd, ifname, &info);
    display_info(&info, ifname);

    if (info.has_mode) 
        printf("mode is %d",info.mode);
    if (info.essid_on)
        printf("essid is %s",info.essid);
}


JNIEXPORT jintArray JNICALL Java_CoreBroker_IFUtils_getACInfo
  (JNIEnv * env, jobject jTmp)
{
    int fd;
    apm_info apm;
    int result;
    jintArray jIntArray = (*env)->NewIntArray(env, 9);
    int exists;
    
    /* If APM support of the right version exists in kernel, return zero.
     * Otherwise, return 1 if no support exists, or 2 if it is the wrong
     * version.  *NOTE* The sense of the return value is not intuitive.
     */
    exists = apm_exists();    

    if (exists == 0) {
        if ((fd = apm_open()) < 0)
        {
            printf("apm_open() failed: ");
            // ?? return null; ??
        } else {
            apm_read(&apm);

            (*env)->SetIntArrayRegion(env, jIntArray, 0, 1, (jint*)&apm.apm_version_major);
            (*env)->SetIntArrayRegion(env, jIntArray, 1, 1, (jint*)&apm.apm_version_minor);
            (*env)->SetIntArrayRegion(env, jIntArray, 2, 1, (jint*)&apm.apm_flags);
            (*env)->SetIntArrayRegion(env, jIntArray, 3, 1, (jint*)&apm.ac_line_status);
            (*env)->SetIntArrayRegion(env, jIntArray, 4, 1, (jint*)&apm.battery_status);
            (*env)->SetIntArrayRegion(env, jIntArray, 5, 1, (jint*)&apm.battery_flags);
            (*env)->SetIntArrayRegion(env, jIntArray, 6, 1, (jint*)&apm.battery_percentage);
            (*env)->SetIntArrayRegion(env, jIntArray, 7, 1, (jint*)&apm.battery_time);
            (*env)->SetIntArrayRegion(env, jIntArray, 8, 1, (jint*)&apm.using_minutes);
            
            apm_close(fd);
        }
    } //else
      // ?? return null; ??
    return jIntArray;
}




JNIEXPORT jobjectArray JNICALL Java_CoreBroker_IFUtils_getIFAddrs
  (JNIEnv * env, jobject jTmp)
{
    struct ifreq *ifr, *ifend;
    unsigned int ina, mask, ip_addr;
    struct ifreq ifreq;
    struct ifconf ifc;
    struct ifreq ifs[5];
    int SockFD;

    int index = 0;
    jclass clazzString = (*env)->FindClass(env,"java/lang/String");
    jobjectArray jStringArrayIpAddrs = (*env)->NewObjectArray(env, 10, clazzString, NULL);
    struct in_addr addr;
    const char ipAddrString[255];
    jstring jIpAddrString;

    int closeID;

    SockFD = socket(AF_INET, SOCK_DGRAM, 0);

    if (SockFD < 0) {
      printf("errno = %d\n", errno);
      printf("EPROTONOSUPPORT: %d\n",EPROTONOSUPPORT);
      printf("EMFILE: %d\n",EMFILE);
      printf("ENFILE: %d\n",ENFILE);
      //printf("EACCESS: %d\n",EACCESS);
      printf("ENOBUFS: %d\n",ENOBUFS);
    }

    ifc.ifc_len = sizeof(ifs);
    ifc.ifc_req = ifs;
    if (ioctl(SockFD, SIOCGIFCONF, &ifc) < 0)
    {
        printf("ioctl(SIOCGIFCONF): %m\n");
        return 0;
    }
    ifend = ifs + (ifc.ifc_len / sizeof(struct ifreq));

    for (ifr = ifc.ifc_req; ifr < ifend; ifr++)
    {
        if (ifr->ifr_addr.sa_family == AF_INET)
        {
            ina = ((struct sockaddr_in *)
            &ifr->ifr_addr)->sin_addr.s_addr;
            strncpy(ifreq.ifr_name, ifr->ifr_name,
            sizeof(ifreq.ifr_name));

            /*
             *  info about network card interface
             */
             
              
	      if (strcmp(  ifreq.ifr_name, "usbf")==0)
		continue;
	      //printf("\n name=%s,%x ",ifreq.ifr_name,ifreq.ifr_name);

            if (ioctl(SockFD, SIOCGIFFLAGS, &ifreq) < 0)
                continue;


            /*
             *  info about netmask
             */
            if (ioctl(SockFD, SIOCGIFNETMASK, &ifreq) < 0)
                continue;
            mask = ((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr.s_addr;
            //printf(" mask=%x \n",mask);

            /*
             *  info about IP address
             */
            if (ioctl(SockFD, SIOCGIFADDR, &ifreq) < 0)
                continue;
            ip_addr = ((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr.s_addr;
            addr = ((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr;
            sprintf((char*)ipAddrString,"%s", inet_ntoa(addr));
            jIpAddrString = (*env)->NewStringUTF(env, ipAddrString);
            (*env)->SetObjectArrayElement(env, jStringArrayIpAddrs, index++, jIpAddrString);
        }
    }

    closeID = close(SockFD);

    //printf("sockfd=%d , close sock=%d\n",SockFD,closeID);

    return jStringArrayIpAddrs;
}


JNIEXPORT jobjectArray JNICALL Java_CoreBroker_IFUtils_getNetworkIFInfo
  (JNIEnv * env, jobject jTmp)
{
    struct ifreq *ifr, *ifend;
    unsigned int ina, mask, ip_addr;
    struct ifreq ifreq;
    struct ifconf ifc;
    struct ifreq ifs[5];
    int SockFD;

    int index = 0;
    jclass clazzNetworkIFDescr = (*env)->FindClass(env,"CoreBroker/NetworkIFDescr");
    jobjectArray jclazzNetworkIFDescrArray = (*env)->NewObjectArray(env, 10, clazzNetworkIFDescr, NULL);

    jmethodID mid = (*env)->GetMethodID(env, clazzNetworkIFDescr, "<init>", "(Ljava/lang/String;Ljava/lang/String;ILjava/lang/String;)V");
    jobject objNetworkIFDescr ;
    jstring jIFNameString;

    char* ifname;
    struct wireless_info	info;
    int rc;  
    int skfd;	
    jstring jessidString;
    jint jmode;

    struct in_addr addr;
    const char ipAddrString[255];
    jstring jIpAddrString;


    SockFD = socket(AF_INET, SOCK_DGRAM, 0);
    ifc.ifc_len = sizeof(ifs);
    ifc.ifc_req = ifs;
    if (ioctl(SockFD, SIOCGIFCONF, &ifc) < 0)
    {
        printf("ioctl(SIOCGIFCONF): %m\n");
        return 0;
    }
    ifend = ifs + (ifc.ifc_len / sizeof(struct ifreq));

    for (ifr = ifc.ifc_req; ifr < ifend; ifr++)
    {
        if (ifr->ifr_addr.sa_family == AF_INET)
        {
            ina = ((struct sockaddr_in *)
            &ifr->ifr_addr)->sin_addr.s_addr;
            strncpy(ifreq.ifr_name, ifr->ifr_name,
            sizeof(ifreq.ifr_name));

            /*
             *  info about network card interface
             */
	    if (strcmp(  ifreq.ifr_name, "usbf")==0)
		continue;
            //printf("\n name=%s,%x ",ifreq.ifr_name,ifreq.ifr_name);
              
            jIFNameString = (*env)->NewStringUTF(env, ifreq.ifr_name);
            ifname = strdup((*env)->GetStringUTFChars(env,jIFNameString, 0));
              

            if (ioctl(SockFD, SIOCGIFFLAGS, &ifreq) < 0)
                continue;


            /*
             *  info about netmask
             */
            if (ioctl(SockFD, SIOCGIFNETMASK, &ifreq) < 0)
                continue;
            mask = ((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr.s_addr;
            //printf(" mask=%x \n",mask);

            /*
             *  info about IP address
             */
            if (ioctl(SockFD, SIOCGIFADDR, &ifreq) < 0)
                continue;
            ip_addr = ((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr.s_addr;
            addr = ((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr;
            sprintf((char*)ipAddrString,"%s", inet_ntoa(addr));
            jIpAddrString = (*env)->NewStringUTF(env, ipAddrString);

            // (*env)->SetObjectArrayElement(env, jStringArrayIpAddrs, index++, jIpAddrString);


            /*
             *  info about wireless  
             */
            if((skfd = iw_sockets_open()) < 0)
            {
                //exit(-1);
                jmode = -1;
                jessidString = (*env)->NewStringUTF(env, "");
            } else {
                rc = get_info(skfd, ifname, &info);

                if (info.has_mode) {
                    jmode = info.mode;
                } else
                    jmode = 3;

                if (info.essid_on) {
                    jessidString = (*env)->NewStringUTF(env, info.essid);
                } else {
                    jessidString = (*env)->NewStringUTF(env, "");
                } 
                close(skfd);    
            }

            objNetworkIFDescr = (*env)->NewObject(env, clazzNetworkIFDescr, mid, jIFNameString, jIpAddrString, jmode, jessidString);
        
            (*env)->SetObjectArrayElement(env, jclazzNetworkIFDescrArray, index++, objNetworkIFDescr);

            //(*env)->ReleaseStringUTFChars(env,jurl, srvurl);
            //(*env)->DeleteLocalRef(env, objServiceURL);

        }
    }

    close(SockFD);

    return jclazzNetworkIFDescrArray;
}

JNIEXPORT jint JNICALL Java_CoreBroker_IFUtils_iproute
  (JNIEnv * env, jobject jTmp, jstring jIpGW)
{
    char* argDel[2];
    char* argAdd[4];
    const char *ipGW = (*env)->GetStringUTFChars(env,jIpGW,0);
    int ret;

    argDel[0] = strdup("delete");
    argDel[1] = strdup("default");
    ret = do_iproute(2, argDel);

    //printf("return1 is %d ",ret);

        
    argAdd[0] = strdup("add");
    argAdd[1] = strdup("default");
    argAdd[2] = strdup("via");
    argAdd[3] = strdup(ipGW);
    ret = do_iproute(4, argAdd);

    //printf("return2 is %d ",ret);

    if (ret == 0)
        return 0;
    else 
        return -1;
}
