package WSAMI;

import CoreBroker.CBException;

import org.w3c.dom.*;
import javax.xml.parsers.*;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.io.IOException;

public class WSAMIAbstractDocument extends WSAMIDocument { 

    private String iface;
    private String conversation;
    private String[] connectorQoS;

    public WSAMIAbstractDocument() {}
   
    public WSAMIAbstractDocument(String XSDfilesPath, String filename) 
            throws CBException {
            
        Document document = getDocumentFromFilename(XSDfilesPath, filename);
            
        Element root = document.getDocumentElement();
        //Node abstractNode = document.getElementsByTagName("Abstract").item(0);
        //this.setName(abstractNode.getAttributes().item(0).getNodeValue());
        
        Node interfaceNode = document.getElementsByTagName("Interface").item(0);
        this.setInterface(interfaceNode.getAttributes().item(0).getNodeValue());
        
        Node convNode = document.getElementsByTagName("Conversation").item(0);
        if (convNode != null)
            this.setConversation(convNode.getAttributes().item(0).getNodeValue());
        
        NodeList connectorsNode = document.getElementsByTagName("ConnectorQoS");
        if (connectorsNode.getLength() != 0) {
            // must have only zero or one ConnectorQoS tag
            Node connectorNode = connectorsNode.item(0);
            NodeList qosNode = connectorNode.getChildNodes();
           
            java.util.Vector qosCritVector = new java.util.Vector();
            for (int i=0; i<qosNode.getLength(); i++)
                if (qosNode.item(i).getNodeName().equals("QoSCriterion"))
                    qosCritVector.add(qosNode.item(i).getAttributes().item(0).getNodeValue());
            String[] qosCrit = new String[qosCritVector.size()];
            for (int i=0; i<qosCritVector.size(); i++)
                qosCrit[i] = (String)qosCritVector.get(i);

            this.setConnectorQoS(qosCrit);
        } else
            this.setConnectorQoS(null);
        
    }
   
   
    public void setInterface(String _interface) {
        iface = _interface;
    }

    public void setConversation(String _conversation) {
        conversation = _conversation;
    }

    public void setConnectorQoS(String[] _connectorQoS) {
        if (_connectorQoS != null) {
            connectorQoS = new String[_connectorQoS.length];
            for (int i=0; i<_connectorQoS.length; i++)
                connectorQoS[i] = _connectorQoS[i];
        } else
            connectorQoS = null;
    }

    public String getInterface() {
        return iface;
    }

    public String getConversation() {
        return conversation;
    }

    public String[] getConnectorQoS() {
        return connectorQoS;
    }

      
    public Document getXML() {
        Document d = super.getXML();
        if (d != null) {
            Element definition = (Element)d.getElementsByTagName("Definition").item(0);

            Element abstractEl = d.createElement("Abstract");
            abstractEl.setAttribute("name",name);
            definition.appendChild(abstractEl);

            Element interfaceEl = d.createElement("Interface");
            interfaceEl.setAttribute("hrefSchema", iface);
            abstractEl.appendChild(interfaceEl);

            if (conversation != null) {
                Element convEl = d.createElement("Conversation");
                convEl.setAttribute("hrefSchema", conversation);
                abstractEl.appendChild(convEl);
            }

            if (connectorQoS != null && connectorQoS.length > 0) {
                Element connectorEl = d.createElement("ConnectorQoS");
                abstractEl.appendChild(connectorEl);
                for (int i=0; i<connectorQoS.length; i++) {
                    Element qosEl = d.createElement("QoSCriterion");
                    qosEl.setAttribute("name", connectorQoS[i]);
                    connectorEl.appendChild(qosEl);
                }
            }

            return d;
      } else 
            return null;
    }
   
}
